#
#       HBS REDUCTION SUB ROUTINE
#
#      Prescan and prefit for template images (wavelength derection)
#                                                 : hbs1to1x.cl
#
#
#      The template images must be ordered by star position from
#      UPPER to LOWER diaph.
#
#
#

procedure hbs1to1x

begin

  file fn_root
  file fn_rootnum
  file fn_scanp, fn_scanw
  string fn_fitp[2], fn_fitw[2], fn_tmp
  file fn_scan, infn
  file fn_y2 = "hbs1to1fitx.dat"
  file tmp_fn, tmp_list, tmp_tbl1
  int i, j, k, l, m
  int flag_old = 0
  int nx, ny
  int nx0 = 20 # start point of scanning from this pixel in X-direction (80)
  real dx = 5
  real y[2], y1, y2, y3
#  int n_image = 20 # The extetion will be 00, 01,..., nn # need special obs.
  int n_image = 10 # The extetion will be 00, 01,..., nn # need special obs.
#  int n_image = 9 # The extetion will be 00, 01,..., nn # 
#  int n_image = 1 # The extetion will be 00, 01,..., nn # 
  real c[6]

  string fn_stack[300]
  string hbslist[50]
  string shorttyp[50], title[50]
  string exptime[50], diaph[50], filter[50]
  string slit[50]
  string listoflist
  string buf = ""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  int n_list

  int n_version      # IRAF version (210 or 211)

#----- initial settings

  listoflist = envget( "_rootlist" )
  fn_fitp[1] = envget( "_1to1we" )
  fn_fitp[2] = envget( "_1to1wo" )

  tmp_fn = mktemp( "hbs.tmp.image." )
  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_tbl = mktemp( "hbs.tmp.tbl." )

  print( "" )
  print( "-------------------------------------" )
  print( " X-direction Fit for template images" )
  print( "-------------------------------------" )
  print( "" )


  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    bye
  }


  for( i=1; i<=2; i+=1 )
  {
    fn_fitw[i] = fn_root//"width."//i//".tbl"
    if( access( fn_fitp[i] ) == 1 )
      delete( fn_fitp[i], ver- )
    if( access( fn_fitw[i] ) == 1 )
      delete( fn_fitw[i], ver- )
  }

#----- IRAF version check

  buf1 = envget( "version" )
  i = stridx( "V", buf1 )
  buf2 = substr( buf1, i, ( i + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#----- Scanning the list file

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      break
    }
    if( buf2 == "DAR" || buf2 == "BIA" || buf2 == "NEA" )
      next

    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- make averaged images

  if( access( tmp_list ) == 1 )
    delete( tmp_list, ver- )

  printf( "-----\n" )
  for( i=1; i<=n_list; i+=1 )
    printf( "%2d %s %s %s\n", i, hbslist[i], title[i], diaph[i] )
  printf( "-----\n" )
  printf( "The lists must be ordered by star positions from UPPER to LOWER diaph\n" )

  printf( "Input head name of averaged image >> " )
  l = scan( fn_root )

  for( k=0; k<=n_image; k+=1 )
  {
    buf = "00"//str(k)
    fn_rootnum = fn_root//substr( buf, ( strlen(buf) - 1 ), strlen( buf) )//"."
    fn_scan = fn_rootnum//"tr"
    if( access( fn_scan//".imh" ) == 1 && flag_old < 2 )
    {
      printf( "'%s' already exists. Use this? [y/n, To all:Y/N] >> ", fn_scan )
      i = scan( buf )
      while( buf != "y" && buf != "Y" && buf != "n" && buf != "N" )
        i = scan( buf )
      if( buf == "y" )
        flag_old = 1
      if( buf == "Y" )
        flag_old = 2
      if( buf == "N" )
        flag_old = 3
	
      if( flag_old == 0 || flag_old == 3 )
        imdelete( fn_scan, ver- )
    }
    if( flag_old == 1 )
    {
      flag_old = 0
      next
    }
    if( flag_old == 2 )
      next

    printf( "Select for %d/%dth group >> ", k, n_image )

    l = scan( m )
    while( m < 1 || m > n_list )
      l = scan( m )
    sed( 's/fit\.imh/tr/', hbslist[m], >> tmp_list )
#    sed( 's/fit\.imh/px/', hbslist[m], >> tmp_list )
    imsum( "@"//tmp_list, fn_scan, hparam="EXPTIME", pixtype="real",\
           calctype="real", option="sum", ver- )
    delete( tmp_list, ver- )
  }

#---- main routine

  for( k=0; k<=n_image; k+=1 )
  {
    buf = "00"//str(k)
    fn_rootnum = fn_root//substr( buf, ( strlen(buf) - 1 ), strlen( buf) )//"."
    fn_scan = fn_rootnum//"tr"
    fn_scanp = fn_rootnum//"pek."
    fn_scanw = fn_rootnum//"wid."

    if( access( fn_scan//".imh" ) == 0 )
    {
      printf( "Cannot find '"//fn_scan//"'. " )
      exit( -1 )
    }

prescanning:

    boxcar( fn_scan, tmp_fn, dx, 1, boundary="nearest" )

#---- search four spectra and define initial constants

    if( access( "database/id"//fn_scan ) )
        delete( "database/id"//fn_scan, ver- )
    if( access( fn_y2 ) )
        delete( fn_y2, ver- )

    print( "mark the 1st and 2nd peaks in order with [m]-key" )

    identify( fn_scan, section="middle column", databas='database',\
              coordli='home$etc/null.dat', nsum=500, match=20.,\
              maxfeat=2, fwidth=10., cradius=15., functio='legendre',\
              order=2, niterate=10, low_rej=3., high_rej=3.,\
              grow=0.,ftype='emission', mode='ql',thresho=5,\
              autowrite+ )

    list = "database/id"//fn_scan

    for( i=1; i<=2; i+=1 )
    {
      while( fscan( list, buf1, buf, buf, buf, buf, buf ) != 6 ){}
      y[i] = real( buf1 )
#      printf( "%6.2f\n", y[i], >> fn_y2 )
    }
    delete( "database/id"//fn_scan )

#---- scan two spectra

    imgets( fn_scan, 'i_naxis1' )
    nx = int( imgets.value )
    imgets( fn_scan, 'i_naxis2' )
    ny = int( imgets.value )

    if( access( tmp_tbl ) == 1 )
      delete( tmp_tbl, ver- )

    for( j=1; j<=2; j+=1 )
    {
      if( access( fn_scanp//j ) == 1 )
        delete( fn_scanp//j, ver- )
#      if( access( fn_scanw//j ) == 1 )
#        delete( fn_scanw//j, ver- )
    }

    if( n_version == 210 )
    { 
       wfits( tmp_fn, tmp_tbl, bscale=1.0, bzero=0.0, long_header-,
              short_header-, bitpix=-32, scale-, autoscale- )
    }
    else
    {
      if( n_version == 211 )
      {
         wfits( tmp_fn//".imh", tmp_tbl, bscale=1.0, bzero=0.0,\
                long_header-, short_header-, bitpix=-32, scale-,\
                fextn="", autoscale- )
      }
      else
      {
        printf( "IRAF version erorr (%d)\n", n_version )
        bye
      }
    }

    hbsscan2p( tmp_tbl, y[1], y[2], nx0, nx, fn_scanp )
#    hbsscan2w( tmp_tbl, y[1], y[2], 0.05, nx0, nx, fn_scanw )

    delete( tmp_tbl, ver- )

    imdelete( tmp_fn, ver- )


#---- search plot files

    for( i=1; i<=2; i+=1 )
    {
      infn = fn_scanp//i
      if( access( infn ) == 0 )
      {
        print( "cannnot find "//infn )
        bye
      }
#      infn = fn_scanw//i
#      if( access( infn ) == 0 )
#      {
#        print( "cannnot find "//infn )
#        bye
#      }
    }
 
#----- Scanning the list file

    for( i=1; i<=2; i+=1 )
    {

    #---- peak ----
      infn = fn_scanp//i

      curfit( infn, function="legendre", weighting="uniform",\
              order=6, interactive+, axis=1, listdata-, ver-,\
              calctype="double", power+, device="stdgraph", cursor="",\
              > tmp_tbl )

      list = tmp_tbl

      while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5,\
             buf6, buf7 ) != EOF )
      {
        if( strlen( buf ) != 1 )
          next
        if( buf == "#" && strlen( buf1 ) != 1 )
          next
        if( buf != "#" )
        {
          j = int( buf )
          if( j > 0 )
          {
            c[j] = real( buf1 )
          }
        }
        else
        {
          j = int( buf1 )
          if( j > 0 )
          {
            c[j] = real( buf2 )
          }
        }
      }

      fn_tmp = fn_fitp[i]
      printf( "%12.9e %12.9e %12.9e %12.9e %12.9e %12.9e\n",\
              c[1], c[2], c[3], c[4], c[5], c[6], >> fn_tmp )

      delete( tmp_tbl, ver- )

    #---- width ----
#      infn = fn_scanw//i

#      curfit( infn, function="legendre", weighting="uniform",\
#              order=6, interactive+, axis=1, listdata-, ver-,\
#              calctype="double", power+, device="stdgraph", cursor="",\
#              > tmp_tbl )

#      list = tmp_tbl

#      while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5,\
#                    buf6, buf7 ) != EOF )
#      {
#        if( strlen( buf ) != 1 )
#          next
#        j = int( buf )
#        if( j > 0 )
#        {
#          c[j] = real( buf1 )
#        }
#      }

#      fn_tmp = fn_fitw[i]
#      printf( "%12.9e %12.9e %12.9e %12.9e %12.9e %12.9e\n",\
#              c[1], c[2], c[3], c[4], c[5], c[6], >> fn_tmp )

#      delete( tmp_tbl, ver- )
    }
  }
  
end

