#
#       HBS REDUCTION SUB ROUTINE
#
#       'One dimensional,' 'Identify' and 'dispcor' for comparison images
#
#                                                      : hbscompdc.cl
#
#       1997/04/18      K.Kawabata   ver 1.00
#       1997/05/01      K.Kawabata   ver 1.01 one-dimensional
#
#
procedure hbscompdc( ref_dir1 )

string ref_dir1 { prompt = "Directry where prescanned image exists" }


begin

  int c, i, j
  file fn, fn1, fn2
  file buf, buf1, buf2, buf3, buf4
  file buf5, buf6, buf7, buf8, buf9, buf10
  file fn_scan = "prescan1.px"
  string rdir1
  string fitsec="middle line"
  string wavelist
  file tmp_list
  real sw1 = 3600.0
  real ew1 = 8900.0
  real dw1 = 5.0
	
  
  print( "" )
  print( "---------------------------------------------" )
  print( " Dispersion correction for comparison images" )
  print( "---------------------------------------------" )
  print( "" )

  rdir1 = ref_dir1
  tmp_list = mktemp( "hbs.tmp.list." )

#----- some confirmation about prescanned image

  if( access( rdir1//"/"//fn_scan//".imh" ) == 0 )
  {
    print( "image '"//rdir1//"/"//fn_scan//"' not found" )
    bye
  }

  if ( access( rdir1//'/database/ap'//fn_scan) == 0 )
  {
    print( "image '"//rdir1//"/"//fn_scan//"' may not be prescanned" )
    bye
  }

  if( access( 'database/ap'//fn_scan ) == 1 )
  {
    print( rdir1//" is this directory ? (y/n)" )
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "y" )
      goto nextstep1
    delete( 'database/ap'//fn_scan, ver- )
  }

  if( access( 'database' ) == 0 )
    mkdir( "database" )
  copy( rdir1//'/database/ap'//fn_scan, "./database/", ver- )


nextstep1 :

#----- Delete old parameters of comparison images

  for( i=1; i<=4; i+=1 )
  {
    for( j=1; j<=4; j+=1 )
    {
      fn1 = "COMP."//i//"t.px."//j
      if( access( 'database/id'//fn1 ) )
        delete( 'database/id'//fn1, ver- )
      if( access( 'database/fc'//fn1 ) )
        delete( 'database/fc'//fn1, ver- )
    }
  }

  sections( "COMP.*.dc.?.imh,COMP.*.px.?*.imh", option="fullname", > tmp_list )
  if( sections.nimages > 0 )
  {
    print( "There are old proccessed COMP images..." )
    type( tmp_list )
    print( "Deleted." )
    imdelete( "@"//tmp_list )
  }
  delete( tmp_list, ver- )


#----- search COMP images

  for( i=1; i<=4; i+=1 )
  {
    if( i == 1 || i == 3 )
      next


    fn = "COMP."//i//"t."
    fn1 = fn//"px"
    buf1 = fn//"px.1"
    buf2 = fn//"px.2"
    buf3 = fn//"px.3"
    buf4 = fn//"px.4"
    buf5 = fn//"dc.1"
    buf6 = fn//"dc.2"
    buf7 = fn//"dc.3"
    buf8 = fn//"dc.4"

    buf9 = buf1//","//buf2//","//buf3//","//buf4
    buf10 = buf5//","//buf6//","//buf7//","//buf8

    if( access( fn1//".imh" ) == 0 )
      next

#----- select type

    print( "image type  [h]:Hg-Ne [i]:nebula [n]:TI-CCD >>" )
    c = scan( buf )
    while( buf != "h" && buf != "i" && buf != "n" )
      c = scan( buf )
    if( buf == "h" )
      wavelist = "home$etc/Hg-Ne.dat"
    if( buf == "i" )
      wavelist = "home$etc/hbsnebula.dat"
    if( buf == "n" )
      wavelist = "home$etc/hbscryocam1.dat"
    type( wavelist )

#----- aperture extract

    print( "Now extracting..." )

    hbsapext( fn1, fn_scan, buf1, buf2, buf3, buf4 )

    hedit( buf1, "HISTORY5", "spectrum extracted refering to "\
           //fn_scan, delete-, verify-, show-, update+ )
    hedit( buf2, "HISTORY5", "spectrum extracted refering to "\
           //fn_scan, delete-, verify-, show-, update+ )
    hedit( buf3, "HISTORY5", "spectrum extracted refering to "\
           //fn_scan, delete-, verify-, show-, update+ )
    hedit( buf4, "HISTORY5", "spectrum extracted refering to "\
           //fn_scan, delete-, verify-, show-, update+ )

#    display( buf1, 1, erase+, xmag=0.6, ymag=0.6, xcenter=0.5,\
#             ycenter=0.10, zscale+, zrange+ )
#    display( buf2, 1, erase-, xmag=0.6, ymag=0.6, xcenter=0.5,\
#             ycenter=0.20, zscale+, zrange+ )
#    display( buf3, 1, erase-, xmag=0.6, ymag=0.6, xcenter=0.5,\
#             ycenter=0.30, zscale+, zrange+ )
#    display( buf4, 1, erase-, xmag=0.6, ymag=0.6, xcenter=0.5,\
#             ycenter=0.40, zscale+, zrange+ )

    imgets( buf1, 'i_naxis2' )
    c = int( imgets.value )
    blkavg( buf1, buf1, 1, c, option="sum" )
    imgets( buf2, 'i_naxis2' )
    c = int( imgets.value )
    blkavg( buf2, buf2, 1, c, option="sum" )
    imgets( buf3, 'i_naxis2' )
    c = int( imgets.value )
    blkavg( buf3, buf3, 1, c, option="sum" )
    imgets( buf4, 'i_naxis2' )
    c = int( imgets.value )
    blkavg( buf4, buf4, 1, c, option="sum" )
    
#----- identify and reidentify

    print( "[m]:mark feature [f]:fit [d]:delete feature [q]:quit" )

    identify( buf1, section=fitsec, databas='database',\   
              coordli=wavelist, nsum=1, match=20.,\
              maxfeat=10, fwidth=5., cradius=10., functio='chebyshev',\
              order=4, niterate=10, low_rej=3., high_rej=3., autowrite+,\
              grow=0.,ftype='emission', mode='ql',thresho=5. )

    reidentify( reference=buf1, images=buf2, answer+, interac-,\
                section=fitsec, newaps-, override+, refit-,\
                trace-, nsum=1, shift=0., nlost=0, cradius=10,\
                threshold=5., addfeatures-, coordlist=wavelist,\
                match=20, maxfeatures=10, minsep=5, databas='database',\   
                logfiles="", plotfile="", ver+, graphics="stdgraph",\
                cursor="", mode='ql' )
    reidentify( reference=buf1, images=buf3, answer+, interac-,\
                section=fitsec, newaps-, override+, refit-,\
                trace-, nsum=1, shift=0., nlost=0, cradius=10,\
                threshold=5., addfeatures-, coordlist=wavelist,\
                match=20, maxfeatures=10, minsep=5, databas='database',\   
                logfiles="", plotfile="", ver+, graphics="stdgraph",\
                cursor="", mode='ql' )
    reidentify( reference=buf1, images=buf4, answer+, interac-,\
                section=fitsec, newaps-, override+, refit-,\
                trace-, nsum=1, shift=0., nlost=0, cradius=10,\
                threshold=5., addfeatures-, coordlist=wavelist,\
                match=20, maxfeatures=10, minsep=5, databas='database',\   
                logfiles="", plotfile="", ver+, graphics="stdgraph",\
                cursor="", mode='ql' )

#----- fitcoord and transformation

nextstep2:

    hedit( buf1, "REFSPEC1", buf1, delete-, add+,\
               verify-, show-, update+ )
    dispcor( buf1, buf5, linearize+, database="database",\
             table="", w1=sw1, w2=ew1, dw=dw1, nw=INDEF, log-,
             flux+, samedisp-, global-, ignoreaps+, confirm-, listonly-,\
             logfile="", ver- )

    hedit( buf2, "REFSPEC1", buf2, delete-, add+,\
               verify-, show-, update+ )
    dispcor( buf2, buf6, linearize+, database="database",\
             table="", w1=sw1, w2=ew1, dw=dw1, nw=INDEF, log-,
             flux+, samedisp-, global-, ignoreaps+, confirm-, listonly-,\
             logfile="", ver- )

    hedit( buf3, "REFSPEC1", buf3, delete-, add+,\
               verify-, show-, update+ )
    dispcor( buf3, buf7, linearize+, database="database",\
             table="", w1=sw1, w2=ew1, dw=dw1, nw=INDEF, log-,
             flux+, samedisp-, global-, ignoreaps+, confirm-, listonly-,\
             logfile="", ver- )

    hedit( buf4, "REFSPEC1", buf4, delete-, add+,\
               verify-, show-, update+ )
    dispcor( buf4, buf8, linearize+, database="database",\
             table="", w1=sw1, w2=ew1, dw=dw1, nw=INDEF, log-,
             flux+, samedisp-, global-, ignoreaps+, confirm-, listonly-,\
             logfile="", ver- )

  }

end
