#
#       HBS REDUCTION SUB ROUTINE
#
#       Make a coordinate for distortion correction : hbsfitcoord.cl
#
#       1996/08/01      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1996/12/05      K.Kawabata  Ver.0.11  no input list, create slice data
#

procedure hbsfitcoord

begin

  file fn = "distort.tr"
  file cor_fn = "distort.cc"
  file tmp_list1
  file tmp_fn1
  file hbslist = "list.up.d"
  int c, i
  string buf = ""
  string buf1
  string fitsec = "middle column"
  file fn_slice = "hbsslice.dat"
  file fn_shift = "hbs1234shift.dat"
  string fitysection = "middle column"
  real y[4]
  real x1[4], x2[4], dx[4], dx1[4], dx2[4]
  real ymin[4], ymax[4]
  int ydim = 31
  int flag_ident = 1 # identify 4 spectra?
  
  tmp_list1 = mktemp( "hbs.tmp.list." )
  tmp_fn1 = mktemp( "hbs.tmp.image." )

  if( access( cor_fn//".imh" ) )
  {
    print( "old distorted image '"//cor_fn//"' exists. delete ? (y/n)" )
    c = scan( buf )
    while( c != 1 || buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "y" )
      imdelete( cor_fn, ver- )
  }

  if( access( fn//".imh" ) )
  {
    print( "old distortion image '"//fn//"' exists. delete ? (y/n)" )
    c = scan( buf )
    while( c != 1 || buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "n" )
      goto step1
    imdelete( fn, ver- )
  }

  if( access( fn_slice ) )
  {
    print( "old slice data file '"//fn_slice//"' exists. delete? (y/n)" )
    c = scan( buf )
    while( c != 1 || buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "n" )
      bye
    delete( fn_slice, ver- )
  }

  if( access( fn_shift ) )
  {
    print( "old slice data file '"//fn_shift//"'exists. delete? (y/n)" )
    c = scan( buf )
    while( c != 1 || buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "n" )
      bye
    delete( fn_shift, ver- )
  }


#----- make list of avaraged images
  
  sed( 's/\.d\./\.tr\./', hbslist, > tmp_list1 )
#  sections( "da*.tr.imh", option="fullname", > tmp_list1 )

  
#----- make averaged image

  imcombine( input="@"//tmp_list1, output=fn, combine="average",\
             reject="none", project-, outtype="real",\
             offsets="none", masktype="none", scale="none",\
             zero="none", weight="none", lthreshold=INDEF,\
             hthreshold=INDEF, nlow=1, nhigh=1 )

  delete( tmp_list1, ver- )

step1:

  hedit( fn, "DISPAXIS", "2", add-, delete-, verify-, show-, update+ )

  if (access('database/id'//fn))
    delete('database/id'//fn)

  if (access('database/fc'//fn))
    delete('database/fc'//fn)

  display( fn, 1, fill+ )


#---- identification of four spectrum

  print( "mark the 1, 2, 3, 4 peaks with 'm'" )

  identify( fn, section=fitsec, databas='database',\   
            coordli='home$etc/sp.dat', nsum=20, match=20.,\
            maxfeat=4, fwidth=15., cradius=15., functio='legendre',\
            order=2, niterate=10, low_rej=3., high_rej=3.,\
            grow=0.,ftype='emission', mode='ql',thresho=5, autowrite+ )


#---- re-identification for 2D image

  print( "reidentify automatically? (y/n)" )
  c = scan( buf )
  while( buf != "y" && buf != "n" )
    c = scan( buf )
  if( buf == "y" )
    reidentify( fn, fn, section=fitsec, shift=0.,\
                step=10, nsum=10, cradius=15., nlost=0, refit-, override+,\
                databas='database', coordli='home$etc/sp.dat',\
                thresho=5, mode='ql', plotfile='', ver+,\
                interactive-, answer+)
  else
    reidentify( fn, fn, section=fitsec, shift=0.,\
                step=10, nsum=10, cradius=15., nlost=0, refit-, override+,\
                databas='database', coordli='home$etc/sp.dat',\
                thresho=5, mode='ql', plotfile='', ver+,\
              interactive+, answer+)

#---- fit coordinate

  fitcoords( fn, fitname='', databas='database', interac+,\
             functio='chebyshev', xorder=2, yorder=2,\
             plotfile="",mode='ql' )

#----- confirmation

  transform( fn, cor_fn, fn, databas='database',\
             interpt='linear', mode='ql')
  hbscoord( cor_fn )
  display( cor_fn, 1, fill+ )

  print( "Ok ? (y/n)" )
  c = scan( buf )
  while( c != 1 || buf != "n" && buf != "y" )
    c = scan( buf )
  if( buf == "n" )
  {
    imdelete( cor_fn, ver- )
    goto step1
  }

  if( flag_ident == 1 )
  {
#---- identification of four spectrum in the transformed image.

  if( access( "database/id"//cor_fn ) )
    delete( "database/id"//cor_fn )

  print( "mark the 1, 2, 3, 4 peaks with 'm'" )

  identify( cor_fn, section=fitsec, databas='database',\   
            coordli='home$etc/null.dat', nsum=20, match=20.,\
            maxfeat=4, fwidth=15., cradius=15., functio='legendre',\
            order=2, niterate=10, low_rej=3., high_rej=3.,\
            grow=0.,ftype='emission', mode='ql',thresho=5, autowrite+ )

  list = "database/id"//cor_fn
  for( i=1; i<=4; i+=1 )
  {
    while( fscan( list, buf1, buf, buf, buf, buf, buf ) != 6 ){}
    y[i] = real( buf1 )
    ymin[i] = y[i] - ( ydim - 1.0 ) / 2.0
    ymax[i] = y[i] + ( ydim - 1.0 ) / 2.0
    printf( "%7.2f  %7.2f\n", ymin[i], ymax[i], >> fn_slice )
    printf( "%7.2f  %7.2f\n", ymin[i], ymax[i] )
  }

  delete( "database/id"//fn )

#---- x-shift of four spectrum in one image

  for( i=1; i<=4; i+=1 )
  {
    print( "------ Spectrum "//i )

  #--- identify x-shift for first feature


    imcopy( cor_fn//"[*,"//int(ymin[i]+0.5)//":"//int(ymax[i]+0.5)//"]",
            tmp_fn1, ver- )
    blkavg( tmp_fn1, tmp_fn1, b1=1, b2=ydim, option="sum" )

    if( access( "database/id"//tmp_fn1 ) )
      delete( "database/id"//tmp_fn1 )

    print( "mark with 'm' near the 1st feature (must be emission)" )

    identify( tmp_fn1, section=fitysection, databas="database",\
              coordli="home$etc/null.dat", nsum=1, match=20.,\
              maxfeat=1, fwidth=20., cradius=10., functio="legendre",\
              order=2, niterate=1, low_rej=3., high_rej=3.,\
              grow=0., ftype="emission", mode="ql", threshold=5,
              autowrite+ )

    list = "database/id"//tmp_fn1
    c = fscan( list, buf, buf, buf, buf, buf, buf )
    c = fscan( list, buf, buf, buf, buf, buf, buf )
    while( fscan( list, buf1, buf, buf, buf, buf, buf ) != 6 ){}
    x1[i] = real( buf1 )
    imdelete( tmp_fn1, ver- )

  #--- identify x-shift for second feature


    imcopy( cor_fn//"[*,"//int(ymin[i]+0.5)//":"//int(ymax[i]+0.5)//"]",
            tmp_fn1, ver- )
    blkavg( tmp_fn1, tmp_fn1, b1=1, b2=ydim, option="sum" )

    if( access( "database/id"//tmp_fn1 ) )
      delete( "database/id"//tmp_fn1 )

    print( "mark with 'm' near the 2nd feature (must be absorption)" )

    identify( tmp_fn1, section=fitysection, databas="database",\
              coordli="home$etc/null.dat", nsum=1, match=20.,\
              maxfeat=1, fwidth=20., cradius=10., functio="legendre",\
              order=2, niterate=1, low_rej=3., high_rej=3.,\
              grow=0., ftype="absorption", mode="ql", threshold=5,
              autowrite+ )

    list = "database/id"//tmp_fn1
    c = fscan( list, buf, buf, buf, buf, buf, buf )
    c = fscan( list, buf, buf, buf, buf, buf, buf )
    while( fscan( list, buf1, buf, buf, buf, buf, buf ) != 6 ){}
    x2[i] = real( buf1 )
    imdelete( tmp_fn1, ver- )

  }
  for( i=1; i<=4; i+=1 )
  {
    dx1[i] = x1[i] - x1[1]
    dx2[i] = x2[i] - x2[1]

    dx[i] = 0.5 * ( dx1[i] + dx2[i] )
    
    if( abs( dx1[i] - dx2[i] ) > 3 )
    {
      print( "warning: Difference between x[1] and x[2] is large" )
    }
    printf( "%8.3f %8.3f --> %7.3f\n", x1[i], x2[i], dx[i] )
    printf( "%7.3f\n", dx[i], >> fn_shift )
  }

  }

end
