#
#       HBS REDUCTION SUB ROUTINE
#
#        Recalculate conversion factor by fitting : hbsphfit4.cl
#
#       1998/07/31      K.Kawabata  Ver.2.00
#       2000/02/20      K.Kawabata  Ver.3.00
#

procedure hbsphfit4

begin

  file infn = "HBS.phot.fittab2"
  file outfn1 = "HBS.phot.convf"
  file outfn2 = "HBS.phot.extn"
  string fn_def

  real am
  real cof[7]
  real y
  real lmin
  real lmax
  real dl
  real b1, b1e
  real beta1[7]
  int nord = 6
  int nlist
  string fn_stack[100]
  int flag_use[100]
  real am_stack[100]
  file fn_tpl = "hbs.tpl.i2"
  file fn_err = "hbsphfit2.err"
  real e0 = 3.67e-20 # erg/cm^2/sec/Hz
  real meanmag = 17.0
  real e1
  real expt

  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8
  string tmp_fn1, tmp_fn2, tmp_fn3, tmp_fn4
  file tmp_aho = "aho.tmp"
  real cdelt, crval, crpix
  int npix

  int c, i, j, k
  real lmd, a1, a2
  real a0 = 1.0e+26

  int flag_ver       # 0:no display 1:display
  int n_version      # IRAF version (210 or 211)

#--- initial settings

  fn_def = envget( "_caldir" )//envget( "_defhname" )

  tmp_fn1 = mktemp( "hbs.tmp.fn." )
  tmp_fn2 = mktemp( "hbs.tmp.fn." )
  tmp_fn3 = mktemp( "hbs.tmp.fn." )
  tmp_fn4 = mktemp( "hbs.tmp.fn." )

  print( "" )
  print( "------------------------------" )
  print( " Photmetric Parameter Fitting"
  print( "------------------------------" )
  print( "" )

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  lmin = 0.0
  lmax = 0.0

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    bye
  }

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "RSW" )
    {
      lmin = real( buf3 )
    }
    if( buf2 == "REW" )
    {
      lmax = real( buf3 )
    }
  }
  if( lmin < 1000.0 || lmax < 1000.0 )
  {
    printf( "Read error in '%s'\n", fn_def )
    bye
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      bye
    }
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#--- check template image

  if( access( fn_tpl//".imh" ) == 0 )
  {
    printf( "'%s' not found. Please do 'hbsphfit1' again.\n", fn_tpl )
    bye
  }
  imgets( fn_tpl, "CRPIX1" )
  crpix = real( imgets.value )
  imgets( fn_tpl, "CRVAL1" )
  crval = real( imgets.value )
  imgets( fn_tpl, "CD1_1" )
  cdelt = real( imgets.value )
  imgets( fn_tpl, "i_naxis1" )
  npix = int( imgets.value )

  if( crval < 1000 )
  {
    printf( "Maybe something is wrong in wavelength calibration in %s\n",\
            fn_tpl )
    printf( "Change order in the input-list of 'hbsphfit1', and do again.\n" )
    bye
  }

#----- Scanning the list file

  buf = ""

  if( access( infn ) == 0 )
  {
    print( "Cannot find "//infn//"." )
    bye
  }
  list = infn
  nlist = 0
  while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8 ) == 9  )
  {
    nlist += 1
    fn_stack[nlist] = buf
    flag_use[nlist] = 1
    am_stack[nlist] = real( buf1 )
  }

  if( access( outfn1 ) == 1 || access( outfn1//".imh" ) == 1 )
  {
    imdelete( outfn1, ver- )
  }

  if( access( outfn2 ) == 0 && access( outfn2//".imh" ) == 0 )
  {
    printf( "Extinction spectrum '%s' not found.", outfn2 )
    bye
  }

  if( access( fn_err ) == 1 )
    delete( fn_err, ver- )
  if( access( tmp_fn1 ) == 1 )
    delete( tmp_fn1, ver- )
  if( access( tmp_fn2 ) == 1 )
    delete( tmp_fn2, ver- )
  if( access( tmp_fn3 ) == 1 )
    delete( tmp_fn3, ver- )
  if( access( tmp_fn4 ) == 1 )
    delete( tmp_fn4, ver- )

#----- Check and reject bad data

  checkdata:

  lmd = 5500.0
  while( lmd >= crval && lmd <= ( crval + ( npix - 1 ) * cdelt ) )
  {
    printf( "y - airmass dependence at %6.1fA. Check graph and press [q].\n",\
            lmd )
    list = infn
    for( k=1; k<=nlist; k+=1 )
    {
    #--- read input list
      if( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8 ) != 9 )
        bye
      if( flag_use[k] == 1 )
      {
        am = real( buf1 )
        cof[1] = real( buf2 )
        cof[2] = real( buf3 )
        cof[3] = real( buf4 )
        cof[4] = real( buf5 )
        cof[5] = real( buf6 )
        cof[6] = real( buf7 )
        cof[7] = real( buf8 )
 
        y = cof[1] + cof[2]*lmd + cof[3]*lmd*lmd + cof[4]*(lmd**3)\
             + cof[5]*(lmd**4) + cof[6]*(lmd**5) + cof[7]*(lmd**6)

        printf( "%7.4f  %10e\n", am, y, >> tmp_fn1 )
      }
    }
    curfit( tmp_fn1, function="legendre", weighting="uniform",\
            order=1, interactive+, axis=1, listdata-, ver-,\
            calctype="double", power+, device="stdgraph", cursor="",\
#	    >> dev$null )
	    > tmp_fn2 )
    delete( tmp_fn1, ver- )
    delete( tmp_fn2, ver- )

    printf( "[0]: continue [1]:reject data [2]:plot at other wavelength >> " )
    c = scan( buf )
    while( buf != "0" && buf != "1" && buf != "2" )
      c = scan( buf )
    if( buf == "0" )
    {
      lmd = 0.0
      next
    }
    if( buf == "2" )
    {
      printf( "Input wavelength in angstrom >> " )
      c = scan( lmd )
      while( lmd < crval || lmd > ( crval + ( npix - 1 ) * cdelt ) )
        c = scan( buf )
      next
    }
    printf( "\nData  Airmass\n" )
    for( i=1; i<=nlist; i+=1 )
      printf( "  %2d   %6.3f\n", i, am_stack[i] )

    printf( "Input data number to be rejected >> " )
    c = scan( k )
    while( k < 1 || k > nlist )
      c = scan( k )
    flag_use[k] = 0
  }


#----- main routine

  i = 1;
  lmd = lmin
  dl = 10.0 * cdelt

  while( lmd <= (lmax+1) )
  {
    printf( "%6.1f ", lmd )

    list = infn
    for( k=1; k<=nlist; k+=1 )
    {

  #--- read input list

      if( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8 ) != 9 )
        bye
      if( flag_use[k] == 1 )
      {
        am = real( buf1 )
        cof[1] = real( buf2 )
        cof[2] = real( buf3 )
        cof[3] = real( buf4 )
        cof[4] = real( buf5 )
        cof[5] = real( buf6 )
        cof[6] = real( buf7 )
        cof[7] = real( buf8 )
 
        y = cof[1] + cof[2]*lmd + cof[3]*lmd*lmd + cof[4]*(lmd**3)\
             + cof[5]*(lmd**4) + cof[6]*(lmd**5) + cof[7]*(lmd**6)

        printf( "%7.4f  %10e\n", am, y, >> tmp_fn1 )
      }
    }
    curfit( tmp_fn1, function="legendre", weighting="uniform",\
            order=1, interactive-, axis=1, listdata-, ver-,\
            calctype="double", power+, device="stdgraph", cursor="",\
            > tmp_fn2 )
    delete( tmp_fn1, ver- )
#    type( tmp_fn2 )
#    bye

    list = tmp_fn2
    if( n_version == 210 )
    {
      while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5,\
                     buf6, buf7, buf8 ) != EOF )
      {
        if( strlen( buf ) != 1 )
          next
        j = int( buf )
        if( j == 1 )
	{
          b1 = real( buf1 )
#          b1e = real( buf3 )
	}
      }
    }
    else
    {
      while( fscan( list, buf8, buf, buf1, buf2, buf3, buf4, buf5,\
                     buf6, buf7, buf8 ) != EOF )
      {
        if( strlen( buf ) != 1 )
          next
        j = int( buf )
        if( j == 1 )
	{
          b1 = real( buf1 )
#          b1e = real( buf3 )
	}
      }
    }
    delete( tmp_fn2, ver- )
#    printf( "%6.1f  beta1 %s +/- %s   beta2 %s +/- %s\n",\
#            lmd, b1, b1e, b2, b2e, >> fn_err )

    printf( "%7.1f  %10e\n", lmd, b1, >> tmp_fn3 )

    i+=1
    lmd = lmin + (i-1)*dl
  }
  printf( "\n" )

#---- fitting

  printf( "Plotting beta-1. [q]:continue\n" )

  curfit( tmp_fn3, function="legendre", weighting="uniform",\
          order=nord, interactive+, axis=1, listdata-, ver-,\
          calctype="double", power+, device="stdgraph", cursor="",\
          > tmp_fn2 )
#  type( tmp_fn2 )

  list = tmp_fn2
  if( n_version == 210 )
  {
    while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5,\
                   buf6, buf7, buf8 ) != EOF )
    {
      if( strlen( buf ) != 1 || buf == "=" )
        next
      j = int( buf )
      if( j > 0 )
        beta1[j] = real( buf1 )
    }
  }
  else
  {
    while( fscan( list, buf8, buf, buf1, buf2, buf3, buf4, buf5,\
                   buf6, buf7, buf8 ) != EOF )
    {
      if( strlen( buf ) != 1 || buf == "=" )
        next
      j = int( buf )
      if( j > 0 )
        beta1[j] = real( buf1 )
    }
  }
  delete( tmp_fn2, ver- )
  delete( tmp_fn3, ver- )

#--- Making images

  printf( "Now making '%s'...\n", outfn1 )
  imcopy( fn_tpl, outfn1, ver- )

  e1 = e0 * 10**( -0.4*meanmag )
  for( j=1; j<=npix; j+=1 )
  {
    lmd = crval + ( j - crpix ) * cdelt
#    a1 = beta1[1] + beta1[2]*lmd + beta1[3]*lmd*lmd + beta1[4]*(lmd**3)\
#         + beta1[5]*(lmd**4) + beta1[6]*(lmd**5) + beta1[7]*(lmd**6)
    a1 = beta1[1] + beta1[2]*lmd + beta1[3]*lmd*lmd + beta1[4]*(lmd**3)\
         + beta1[5]*(lmd**4) + beta1[6]*(lmd**5)

    imreplace( outfn1//"["//j//":"//j//"]", ( exp( a1 ) * e1 ) )
  }

  splot( outfn1 )  
  printf( " Plotting conversion factor spectrum [q]:next." )

end
